=head1 NAME

ESP::XMLDump - dump the output of the XML parser

=head1 DESCRIPTION

Dumps the output of the XML parser straight to the screen, in a
verbose format.  Mostly useful for testing, and as an example for
deriving other classes.

=cut

package ESP::XMLDump;

use base 'ESP::ToXML';
use base 'ESP::XMLParser';

# we override the output routine to feed the output of the XML
# converter straight into the XML parser ... nice.
sub output {
  my $self=shift;
  $self->parse_chunk(@_);
}

sub start {
  my $self=shift;
  my $tag=shift;
  my %attrs=%{shift()};
  my @attrseq=@{shift()};
  my $text=shift;

  print "Start: $tag, attrs = {", 
    join(", ", map { "$_ => $attrs{$_}" } keys %attrs),
      "} attrseq = [", join(", ", @attrseq),
        "]; text=\"$text\"\n";
}
sub end {
  my $self=shift;
  my $tag=shift;
  my $text=shift;

  if (defined $text) {
    print ("End: $tag, text=\"$text\"\n");
  } else {
    print ("End: $tag, no text\n");
  }
}
sub text {
  my $self=shift;
  my $text=shift;

  print "Text: \"$text\"\n";
}

1;
