package ESP::XMLParser;

# a really noddy XML parser, to get around the fact that perl doesn't
# provide one as standard.  Should be compatible with HTML::Parser
# (!) v2, which may be quicker if you have it installed.

sub new {
  my $this=shift;
  my $class=ref($this) || $this;
  my $self={};
  bless $self, $class;
  $self->init();
  return $self;
}

sub init {
}

sub parse_fh {
  my $self=shift;
  my $fh=shift;

  while (defined (my $l=<$fh>)) {
    $self->parse_chunk($l);
  }

  $self->parse_end();
}

sub parse_end {
  my $self=shift;

  # make sure we've processed all text
  $self->text($self->{txtbuf});

  delete $self->{txtbuf};
}

sub parse_chunk {
  my $self=shift;

  $self->{linebuf} .= join('', @_);

  while (defined $self->{linebuf} and $self->{linebuf} ne '') {
    # anything that we want to handle specially starts with '<' or '&'
    unless ($self->{linebuf} =~ s!^(.*?)([<&])!!s) {
      # and there's nothing interesting, so remember this one and carry on
      $self->{txtbuf} .= $self->{linebuf};
      undef $self->{linebuf};
      next;
    }

    # OK, got some leading text followed by something
    $self->{txtbuf} .= $1;
    my $start=$2;

    if ($start eq '&') {
      # entity
      die "syntax error: expected entity, got \&$self->{linebuf}"
        unless $self->{linebuf} =~ s!^([a-z]+);!!s;
      $self->{txtbuf} .= $self->decode_entity($1);
      next;
    }

    # tag

    # first of all, this is the end of a load of text, so process
    # it
    $self->text($self->{txtbuf});
    $self->{txtbuf}='';

    if (substr($self->{linebuf}, 0, 3) eq '!--') {
      # comment, for now only allow single-line comments
      die "end-comment missing" unless $self->{linebuf} =~ s!^.*-->!!s;
      next;
    }

    # now what is the tag?
    die "syntax error: expected tag, got <$self->{linebuf}"
      unless $self->{linebuf} =~ s!^([^>]+)>!!s;
    my $tagtext=$1;

    if (substr($tagtext, 0, 1) eq '/') {
      # end tag, not allowed attrs
      die "attrs not allowed in end tag $tagtext" if $tagtext =~ m!\s!s;
      $self->end(substr($tagtext, 1), "<$tagtext>");
      next;
    }

    # start tag or XML empty element tag?
    my $isempty = ($tagtext =~ s!/$!!s);

    # now split up any attrs
    my ($tag, @attrs) = split(/\s+/, $tagtext);

    my (%attrs,@attrseq);
    for my $a (@attrs) {
      my ($name,$val)=split(/=/, $a, 2);
      $attrs{$name}=defined $val ? $val : $name;
      push @attrseq, $name;
    }

    $self->start($tag, \%attrs, \@attrseq, "<$tagtext>");

    if ($isempty) {
      $self->end($tag);
    }
  }
}

{
  my %lut=(
           lt => '<',
           gt => '>',
           amp => '&',
          );
sub decode_entity {
  my $self=shift;
  my $ent=shift;
  return $lut{$ent};
}
}

1;
